#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 Código Sur - Nuestra América Asoc. Civil / Fundación Pacificar.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
cyclope.frontend_views
----------------------
"""
from operator import attrgetter

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.template import Template, Context
from django.contrib.sites.models import Site
from django.contrib.comments.models import Comment
from django.template.loader import render_to_string

import cyclope.utils
from cyclope import settings as cyc_settings
from cyclope.core import frontend
from cyclope.core.collections.models import Collection, Category
from cyclope.models import Menu, MenuItem, Author


class MenuRootItemsList(frontend.FrontendView):
    """A list view of the root MenuItems for a given Menu.
    """
    name='root_items_list'
    verbose_name=_('list of root items for the selected Menu')
    is_default = True
    is_region_view = True
    template = "cyclope/menu_flat_items_list.html"

    def get_response(self, request, req_context, options, content_object):
        menu_items = MenuItem.tree.filter(menu=content_object,
                                          level=0, active=True)
        current_url = request.path_info[1:].split('/')[0]
        return render_to_string(self.template, {
            'menu_items': menu_items,
            'current_url': current_url
        }, req_context)


frontend.site.register_view(Menu, MenuRootItemsList)


class MenuFlatItemsList(frontend.FrontendView):
    """A flat list view of all the MenuItems for a given Menu.
    """
    name='flat_items_list'
    verbose_name=_('flat list of all items for the selected Menu')
    is_region_view = True
    template = "cyclope/menu_flat_items_list.html"

    def get_response(self, request, req_context, options, content_object):
        menu_items = MenuItem.tree.filter(menu=content_object, active=True)
        current_url = request.path_info[1:].split('/')[0]
        return render_to_string(self.template, {
            'menu_items': menu_items,
            'current_url': current_url
        }, req_context)


frontend.site.register_view(Menu, MenuFlatItemsList)

class MenuHierarchyOptions(forms.Form):
    align = forms.ChoiceField(label=_('Collapse style'),
                              choices=(("VERTICAL", _("vertical")),
                                       ("HORIZONTAL", _("horizontal")),
                                       ("ON_CLICK", _("on click")),
                                       ("DISABLED", _("disabled"))),
                              initial="HORIZONTAL")

# TODO(nicoechaniz): refactor this view and CollectionCategoriesHierarchy which share most of their code.
class MenuMenuItemsHierarchy(frontend.FrontendView):
    """A hierarchical list view of the menu items in a menu.
    """
    name='menuitems_hierarchy'
    verbose_name=_('hierarchical list of the items in the selected menu')
    is_region_view = True
    options_form = MenuHierarchyOptions
    template = "cyclope/menu_menuitems_hierarchy.html"

    def get_response(self, request, req_context, options, content_object):
        menu = content_object
        menu_items = MenuItem.tree.filter(menu=menu, level=0, active=True)
        menu_items_list = []
        current_url = request.path_info[1:]
        for item in menu_items:
            menu_items_list.extend(self._get_menuitems_nested_list(item, current_url))
        return render_to_string(self.template, {
            'menu_items': menu_items_list,
            'menu_slug': menu.slug,
            'expand_style': options["align"]
        }, req_context)

    def _get_menuitems_nested_list(self, base_item, current_url=None, name_field='name'):
        """Creates a nested list to be used with unordered_list template tag
        """
        #TODO(nicoechaniz): see if there's a more efficient way to build this recursive template data.
        link_template = Template(
             '<span class="{% if has_children %}has_children{% else %}no_children{% endif %}{% if current_url == menu_item.url%} current{% endif%}{% if menu_item.site_home %} site_home{% endif %}">'
             '{% if menu_item.custom_url %}'
             '   <a href="{{menu_item.url}}">'
             '{% else %}'
             '   <a href="/{{CYCLOPE_PREFIX}}{{menu_item.url}}">'
             '{% endif %}'
             '{{ menu_item.name }}</a></span>'
            )
        nested_list = []
        for child in base_item.get_children().filter(active=True):
            if child.get_descendant_count() > 0:
                nested_list.extend(self._get_menuitems_nested_list(
                    child, current_url, name_field=name_field))
            else:
                name = getattr(child, name_field)
                nested_list.append(link_template.render(
                    Context({'menu_item': child, 'current_url': current_url})))

        name = getattr(base_item, name_field)
        include = link_template.render(
            Context({'menu_item': base_item, 'current_url': current_url,
                     'has_children': base_item.get_descendant_count()}))
        if nested_list:
            return [include, nested_list]
        else:
            return [include]

frontend.site.register_view(Menu, MenuMenuItemsHierarchy)


class MenuItemChildrenOfCurrentItem(frontend.FrontendView):
    """List view of all the sub-items for the currently selected MenuItem
    """
    name='children_list'
    verbose_name=_('list view of children of the current menu item')
    is_default = True
    is_instance_view = False
    is_region_view = True
    template = "cyclope/menu_flat_items_list.html"
    def get_response(self, request, req_context, options):
        base_url = request.path_info[1:].split('/')[0]
        if base_url == '':
            current_item = MenuItem.tree.filter(site_home=True)
        else:
            current_item = MenuItem.tree.filter(url=base_url)

        if current_item:
            children = current_item[0].get_children().filter(active=True)
            return render_to_string(self.template, {'menu_items': children},
                                    req_context)
        else:
            return ''

frontend.site.register_view(MenuItem, MenuItemChildrenOfCurrentItem)


class SiteSearchBox(frontend.FrontendView):
    """Show a small form with an input field to search content
    """
    name='search'
    verbose_name=_('search box')
    is_instance_view = False
    is_region_view = True
    template = "cyclope/site_search_box.html"

    def get_response(self, request, req_context, options):
        return render_to_string(self.template, {}, req_context)

frontend.site.register_view(Site, SiteSearchBox)


class SiteMap(frontend.FrontendView):
    """Show an expanded hierarchical list of all collection and menus
    """
    name='map'
    verbose_name=_('expanded hierarchical list of all collection and menus')
    is_default = True
    is_instance_view = False
    is_content_view = True
    template = "cyclope/site_map.html"

    def get_response(self, request, req_context, options):
        collections_list = []
        for collection in Collection.objects.filter(visible=True):
            category_list = []
            for category in Category.tree.filter(collection=collection, level=0):
                # TODO(diegoM): Change this line when the refactorization is done
                category_list.extend(self._get_categories_nested_list(category))
            if category_list:
                collections_list.extend([collection,category_list])
            else:
                collections_list.append(collection)

        menus_list = []
        for menu in Menu.objects.all():
            menu_items_list = []
            for item in MenuItem.tree.filter(menu=menu, level=0):
                # TODO(diegoM): Change this line when the refactorization is done
                menu_items_list.extend(
                    MenuMenuItemsHierarchy()._get_menuitems_nested_list(item))
            if menu_items_list:
                menus_list.extend([menu.name, menu_items_list])
            else:
                menus_list.append(menu.name)

        return render_to_string(self.template, {
            'collections':collections_list,
            'menus':menus_list,
        }, req_context)

    def _get_categories_nested_list(self, base_category, name_field='name'):

        """Creates a nested list to be used with unordered_list template tag
        """
        #TODO(nicoechaniz): see if there's a more efficient way to build this recursive template data.
        link_template = Template(
            '{% if has_content %}'
              '<a href="{% url category-teaser_list slug %}">'
                 '<span>{{ name }}</span></a>'
            '{% else %} {{ name }}'
            '{% endif %}'
            ' <a href="{% url category_feed slug %}">'
            '<img src="{{ media_url }}images/css/rss_logo.png"/></a>'
            )
        nested_list = []
        for child in base_category.get_children():
            if child.get_descendant_count()>0:
                nested_list.extend(self._get_categories_nested_list(
                    child, name_field=name_field))
            else:
                name = getattr(child, name_field)
                has_content = child.categorizations.exists()
                nested_list.append(link_template.render(
                    Context({'name': name,
                             'slug': child.slug,
                             'has_content': has_content,
                             'media_url':cyc_settings.CYCLOPE_THEME_MEDIA_URL,})))

        name = getattr(base_category, name_field)
        has_content = base_category.categorizations.exists()
        include = link_template.render(
            Context({'name': name,
                     'slug': base_category.slug,
                     'has_content': has_content,
                     'has_children': base_category.get_descendant_count(),
                     'media_url':cyc_settings.CYCLOPE_THEME_MEDIA_URL,}))
        if nested_list:
            return [include, nested_list]
        else:
            return [include]
frontend.site.register_view(Site, SiteMap)


class CommentsListOptions(forms.Form):
    limit_to_n_items = forms.IntegerField(label=_('Items to show'), min_value=1,
                                        initial=5,)


INLINE_CHOICES = (
    ("teaser", _(u"Teaser")),
    ("inline_detail", _(u"Detail")),
    ("labeled_icon", _(u"Labeled icons")),
)

class AuthorDetailOptions(forms.Form):
    show_authored_content = forms.BooleanField(label=_('Show authored content'),
                                               initial=True, required=False)
    inline_view_name = forms.ChoiceField(label=_('Inline type'),
                                         choices=INLINE_CHOICES,
                                         initial="teaser")
    items_per_page = forms.IntegerField(label=_('Items per page'), min_value=1,
                                        initial=cyc_settings.CYCLOPE_PAGINATION['TEASER'],)
    limit_to_n_items = forms.IntegerField(label=_('Limit to N items (0 = no limit)'),
                                          min_value=0, initial=0)
    sort_by = forms.ChoiceField(label=_('Sort by'),
                              choices=(("DATE-", _(u"Date ↓ (newest first)")),
                                       ("DATE+", _(u"Date ↑ (oldest first)")),
                                       ("ALPHABETIC", _(u"Alphabetic"))),
                              initial="DATE-")

class AuthoredMixin(object):

    options_form = AuthorDetailOptions

    def get_queryset(self, content_object):
        "Must return a queryset with all objects 'authored' by content_object"
        return NotImplementedError

    def get_page(self, request, req_context, options, content_object):
        from cyclope.core.collections.frontend_views import SORT_BY
        qs = self.get_queryset(content_object)

        sort_by = options["sort_by"]
        limit = options["limit_to_n_items"] or None
        sort_property, reverse, paginator_class = SORT_BY[sort_by]
        paginator_kwargs = {"per_page": options["items_per_page"]}
        if 'DATE' in sort_by:
            qs.sort(key=attrgetter(sort_property), reverse=reverse)
            if limit:
                qs = qs[:limit]
        elif 'ALPHABETIC' in sort_by:
            paginator_kwargs['on'] = "name"
        paginator = paginator_class(qs, **paginator_kwargs)
        page = cyclope.utils.get_page(paginator, request)
        return page


class AuthorDetail(AuthoredMixin, frontend.FrontendView):
    """Display an author's detail
    """
    name='detail'
    verbose_name=_('detailed view of the selected Author')
    is_default = True
    is_instance_view = True
    is_region_view = False
    is_content_view = True
    template = "cyclope/author_detail.html"

    def get_queryset(self, content_object):
        authored_content = []
        # get related methods like picture_set, article_set, etc.
        related = [attribute for attribute in dir(content_object) \
                   if attribute.endswith('_set')]
        for n in related:
            authored_content.extend(getattr(content_object, n).all())
        return authored_content

    def get_response(self, request, req_context, options, content_object):
        if options['show_authored_content']:
            authored_contents_page = self.get_page(request, req_context, options,
                                                   content_object)
        else:
            authored_contents_page = None

        return render_to_string(self.template, {
            'inline_view_name': options["inline_view_name"],
            'page': authored_contents_page
        }, req_context)


frontend.site.register_view(Author, AuthorDetail)



from django.utils.translation import ugettext_lazy as _

# django

_("Username")

# haystack

_("Search In")

# others

_("Lookup")

_("Share in your social networks")

_("Shortcuts")

_("Collapse tree")

_("Expand tree")

_("and")

_("or make changes")

_("Post")

_("Neutronica theme")

_("One sidebar")

_("left sidebar")

_("footer")

_("Two sidebars")

_("right sidebar")

_("No sidebars")

_("Subscribe to RSS")

_("User")

_("A user with that username already exists.")

_("username")

_("users")

_("sender")

_("Visit website")

_("Categories")

_("Name")

_("Alignment")

_("Text")

_("Date")

_("Share")

_("Review and moderate user comments")

_("alphabetical teaser list of Contact members")

_("Categorization")

_("Classify the content in your website")

_("Modify the menus and site layouts")

_("Header")

_("show the current content for the selected Newsletter")

_("show the header text of the selected Newsletter")

_("toggle editor")

_("on")

_("off")

_("collection categories")

_("Spanish")

_("English")

_("Portuguese")

_("list of Products")

_("price")

_("product")

_("products")

_("Potente theme")

_("One sidebar and four panes")

_("sidebar")

_("second pane")

_("third pane")

_("fourth pane")

_("Translated")

_("article image")

_("article images")

_("attachments")

_("downloads")

_("Codigo Sur, latest news")

_("navigation breadcrumb")

_("Categories for this")

_("category map")

_("%(more_count)s more")



import re
import time
import unittest

import nohang

class TestNohang(unittest.TestCase):
    data = "95756, KURN , 20110311, 2130, -34.00, 151.21, 260, 06.0, -9999.0, -9999.0, -9999.0, -9999.0, -9999.0, -9999, -9999, 07.0, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -9999, -"

    def test_creation(self):

        def function(arg1, arg2, arg3=3):
            return arg1, arg2, arg3

        result, success = nohang.run(function, args=("1", "2"), kwargs={"arg3":"foo"},
                                     wait=0.5)
        self.assertTrue(success)
        self.assertEqual(result[0], "1")
        self.assertEqual(result[1], "2")
        self.assertEqual(result[2], "foo")

        result, success = nohang.run(function, args=("1", "2"), wait=0.1)

        self.assertTrue(success)
        self.assertEqual(result[0], "1")
        self.assertEqual(result[1], "2")
        self.assertEqual(result[2], 3)

    def test_simple(self):
        wait, SLEEP = 0.5, 5
        start = time.time()
        result, success = nohang.run(time.sleep, (SLEEP, ), wait=wait)

        interval = time.time() - start
        self.assertFalse(success)
        self.assertTrue(result is None)

        self.assertTrue(interval < SLEEP)
        self.assertTrue(interval > wait)

    def test_c_code(self):
        rgex = re.compile('^(\d{5}), .+?, (\d{8}), (\d{4}), .+?, .+?,' + 37 * ' (.*?),' + ' (.*?)$')

        result, success = nohang.run(rgex.match, args=(self.data, ), wait=0.1)
        self.assertFalse(success)






#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 Código Sur - Nuestra América Asoc. Civil / Fundación Pacificar.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext_lazy as _

from cyclope.core import frontend
from cyclope import views

from models import *

class MediaDetail(frontend.FrontendView):
    name='detail'
    is_default = True
    is_instance_view = True
    is_content_view = True
    params = {'template_object_name': 'media'}

    def get_response(self, request, req_context, options, content_object):
        return views.object_detail(request, req_context, content_object, **self.params)

class PictureDetail(MediaDetail):
    """Detail view of a Picture.
    """
    verbose_name=_('detailed view of the selected Picture')

frontend.site.register_view(Picture, PictureDetail)


class SoundTrackDetail(MediaDetail):
    """Detail view of an SoundTrack.
    """
    verbose_name=_('detailed view of the selected Audio Track')

frontend.site.register_view(SoundTrack, SoundTrackDetail)


class MovieClipDetail(MediaDetail):
    """Detail view of a MovieClip.
    """
    verbose_name=_('detailed view of the selected Movie Clip')

frontend.site.register_view(MovieClip, MovieClipDetail)


class DocumentDetail(MediaDetail):
    """Detail view of a Document.
    """
    verbose_name=_('detailed view of the selected Document')

frontend.site.register_view(Document, DocumentDetail)


class RegularFileDetail(MediaDetail):
    """Detail view of a regular file.
    """
    verbose_name=_('detailed view of the selected File')

frontend.site.register_view(RegularFile, RegularFileDetail)


class ExternalContentDetail(MediaDetail):
    """Detail view of external content.
    """
    verbose_name=_('detailed view of the selected External Content')

frontend.site.register_view(ExternalContent, ExternalContentDetail)


class FlashMovieDetail(MediaDetail):
    """Detail view of a Flash Movie.
    """
    verbose_name=_('detailed view of the selected Flash Movie')

    def get_response(self, request, req_context, options, content_object):
        p = content_object.flash.url_full
        i = p.rfind("/")+1
        base = p[:i]
        req_context.update({'url_base': base})
        return views.object_detail(request, req_context, content_object,
                                   **self.params)

frontend.site.register_view(FlashMovie, FlashMovieDetail)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 Código Sur - Nuestra América Asoc. Civil / Fundación Pacificar.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from django.conf import settings
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.importlib import import_module

import cyclope.settings as cyc_settings
from cyclope.admin import BaseContentAdmin
from cyclope.core.collections.admin import CollectibleAdmin

from models import Contact, ContactAddress
from forms import ContactForm

class ContactAddressInline(admin.StackedInline):
    model = ContactAddress
    extra = 0
    fields = ('type', 'country', 'region', 'city', 'zip_code', 'street_address', 'phone_number',
              'post_office_box')

class ContactAdmin(CollectibleAdmin, BaseContentAdmin):
    form = ContactForm
    inlines = CollectibleAdmin.inlines + BaseContentAdmin.inlines + [ContactAddressInline]

    list_filter = CollectibleAdmin.list_filter + ('gender', )
    list_display = ('given_name', 'surname', 'gender', 'email', 'web', 'mobile_phone_number')
    search_fields = ('given_name', 'surname', 'email', 'web', 'mobile_phone_number')

    fieldsets = (
        (None, {
            'fields': ('given_name', 'surname', 'birth_date', 'photo', 'gender',
                       'email', 'web', 'mobile_phone_number')
        }),
    )

    # Add ContactProfile admin as inline if defined in settings
    if getattr(settings, 'CYCLOPE_CONTACTS_PROFILE_ADMIN_INLINE_MODULE', False):
        _parts = settings.CYCLOPE_CONTACTS_PROFILE_ADMIN_INLINE_MODULE.split('.')
        _inl = _parts[-1]
        _mod = ".".join(_parts[:-1])
        _module = import_module(_mod)
        profile_inline = getattr(_module, _inl)
        inlines.append(profile_inline)



admin.site.register(Contact, ContactAdmin)



from django.conf.urls import *
from views import preview, send, sent, failed

urlpatterns = patterns(
    '',
    url(r'preview/(?P<id>[\d]+)/$', preview, name='newsletter_preview'),
    url(r'send_test/(?P<id>[\d]+)/$', send, {'test': True},
        name='newsletter_send-test'),
    url(r'send/(?P<id>[\d]+)/$', send, name='newsletter_send'),
    url(r'sent/(?P<id>[\d]+)/$', sent, name='newsletter_sent'),
    url(r'failed/(?P<id>[\d]+)/$', failed, name='newsletter_failed'),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 Código Sur - Nuestra América Asoc. Civil / Fundación Pacificar.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.syndication.views import Feed
from django.shortcuts import get_object_or_404
from django.http import Http404
from django.core.urlresolvers import reverse
from cyclope.core.collections.models import Category
from cyclope.models import SiteSettings
import cyclope.settings as cyc_settings
import cyclope.core.frontend.sites as sites

class WholeSiteFeed(Feed):

    description_template = 'feeds/description.html'

    def title(self, obj):
        return cyc_settings.CYCLOPE_SITE_SETTINGS.global_title

    # TODO(diegoM): It would be better show the first paragraph?
    def item_description(self, content):
        if hasattr(content, 'text'):
            return content.text
        elif hasattr(content, 'description'):
            return content.description

    def item_author(self, content):
        return content.author.name

    def link(self):
        return reverse('whole_site_feed')

    def item_title(self, item):
        #FIXME(diegoM): How to get the right translation of the object_name ?
        return "%s  (%s)" % (item.name, item.get_verbose_name().capitalize())

    def items(self):
        N = cyc_settings.CYCLOPE_RSS_LIMIT
        objs = []
        for ctype in SiteSettings.objects.get().rss_content_types.all():
            objs.extend(list(ctype.model_class().objects.filter(published=True)[:N]))
        return sorted(objs, key=lambda x: x.creation_date, reverse = True)[:N]

class ContentTypeFeed(WholeSiteFeed):

    def title(self, obj):
        return u'%s | %s' % (cyc_settings.CYCLOPE_SITE_SETTINGS.global_title,
                             obj._meta.verbose_name_plural.capitalize())

    def get_object(self, request, object_name):
        for model in sites.site.base_content_types:
            if model.get_object_name() == object_name:
                return model
        raise Http404

    def link(self, model):
        return reverse('content_type_feed', args=[model.get_object_name()])

    def items(self, model):
        N = cyc_settings.CYCLOPE_RSS_LIMIT
        return model.objects.filter(published=True).order_by('-creation_date')[:N]

class CategoryFeed(WholeSiteFeed):

    def title(self, obj):
        return u'%s | %s' % (cyc_settings.CYCLOPE_SITE_SETTINGS.global_title,
                             obj.name.capitalize())

    def get_object(self, request, slug):
        return get_object_or_404(Category, slug=slug)

    def link(self, category):
        return reverse('category_feed', args=[category.slug])

    def items(self, category):
        N = cyc_settings.CYCLOPE_RSS_LIMIT
        objs = []
        for c in category.categorizations.all():
            if c.content_object.published:
              objs.append(c.content_object)
        return sorted(objs, key=lambda x: x.creation_date, reverse = True)[:N]



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2010-2012 Código Sur Sociedad Civil
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    depends_on = (
        ("articles", "0002_auto__del_source__del_author__chg_field_article_author__chg_field_arti"),
    )

    def forwards(self, orm):
        #this migration is resolved in the cyclope.articles 0002 migration
        pass

    def backwards(self, orm):
        # this migration is resolved in the cyclope.articles 0002 migration
        pass


    models = {
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'cyclope.author': {
            'Meta': {'object_name': 'Author'},
            'country': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '250', 'db_index': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'})
        },
        'cyclope.layout': {
            'Meta': {'object_name': 'Layout'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50', 'db_index': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique_with': '()', 'max_length': '50', 'populate_from': 'None', 'db_index': 'True'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'cyclope.menu': {
            'Meta': {'object_name': 'Menu'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'main_menu': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '50', 'db_index': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique_with': '()', 'max_length': '50', 'populate_from': 'None', 'db_index': 'True'})
        },
        'cyclope.menuitem': {
            'Meta': {'object_name': 'MenuItem'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True', 'blank': 'True'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'menu_entries'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'content_view': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'custom_url': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'layout': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Layout']", 'null': 'True', 'blank': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'menu': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'menu_items'", 'to': "orm['cyclope.Menu']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "u'children'", 'null': 'True', 'to': "orm['cyclope.MenuItem']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'site_home': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique_with': '()', 'max_length': '50', 'populate_from': 'None', 'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255', 'db_index': 'True'})
        },
        'cyclope.regionview': {
            'Meta': {'object_name': 'RegionView'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'region_views'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'content_view': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'layout': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Layout']"}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'region': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '100', 'blank': 'True'}),
            'weight': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'})
        },
        'cyclope.sitesettings': {
            'Meta': {'object_name': 'SiteSettings'},
            'allow_comments': ('django.db.models.fields.CharField', [], {'max_length': '4'}),
            'default_layout': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Layout']", 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'global_title': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keywords': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']", 'unique': 'True'}),
            'theme': ('django.db.models.fields.CharField', [], {'max_length': '250'})
        },
        'cyclope.source': {
            'Meta': {'object_name': 'Source'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'link': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '250', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '250', 'db_index': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'})
        },
        'sites.site': {
            'Meta': {'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['cyclope']



# *-- coding:utf-8 --*
"""Configuration for the Neutrona theme templates and regions.

Attributes:
    verbose_name: name of theme to be displayed in the admin interface
    layout_templates: dictionary defining regions for the available templates
"""

from django.utils.translation import ugettext_lazy as _

verbose_name = _('Plantation')

layout_templates = {

    '4_elements.html':
        {
        'verbose_name': _('4 Elements'),
        'regions' : {
            'air': _('air'),
            'water': _('water'),
            'before_fire': _('before fire'),
            'after_fire': _('after fire'),
            'earth': _('earth'),
            }
        },

    '5_elements.html':
        {
        'verbose_name': _('5 Elements'),
        'regions' : {
            'air': _('air'),
            'water': _('water'),
            'love': _('love'),
            'before_fire': _('before fire'),
            'after_fire': _('after fire'),
            'earth': _('earth'),
            }
        },
    'empty.html':
        {'verbose_name': _('Empty'),
         'regions': {}
         },

    'newsletter.html':
        {'verbose_name': _('Newsletter'),
        'regions' : {
            'header': _('header'),
            'before_content': _('before content'),
            'after_content': _('after content'),
            }
         },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010-2012 Código Sur Sociedad Civil
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import time
import json
import unittest
from operator import attrgetter
from collections import defaultdict

from django import forms
from django.test import TestCase
from django.test.utils import setup_test_environment
from django.contrib.sites.models import Site
from django.contrib.auth.models import AnonymousUser
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.template import TemplateSyntaxError, Template, Context
from django import template
from django.db import models
from django.db.models import get_model
import django.contrib.comments

from cyclope.models import SiteSettings, Menu, MenuItem, RelatedContent
from cyclope.models import Layout, RegionView, Author
from cyclope.core import frontend
from cyclope.core.collections.models import *
from cyclope.core.perms.models import CategoryPermission
from cyclope.core.user_profiles.models import UserProfile
from cyclope.templatetags.cyclope_utils import do_join
from cyclope.apps.staticpages.models import StaticPage
from cyclope.apps.articles.models import Article
from cyclope.apps.medialibrary.models import *
from cyclope.apps.polls.models import *
from cyclope.apps.forum.models import *
from cyclope.apps.feeds.models import Feed
from cyclope.apps.dynamicforms.models import DynamicForm

from cyclope.fields import MultipleField
from cyclope.sitemaps import CollectionSitemap, CategorySitemap, MenuSitemap
from cyclope.forms import SiteSettingsAdminForm, LayoutAdminForm, MenuItemAdminForm
from cyclope import themes
from cyclope import templatetags as cyclope_templatetags
from cyclope.templatetags.cyclope_utils import smart_style

DEFAULT_THEME = "neutronix"
DEFAULT_THEME_REGION = "ash"


def create_static_page(name=None):
    if name is None:
        name = 'A page'
    return StaticPage.objects.create(name=name)

def export_fixture(apps, filename=None):
    """
    Return dumpdata from apps. If filename write dumpdata to file.
    """
    from django.core.management.commands.dumpdata import Command as Dumpdata
    cmd = Dumpdata()
    dump = cmd.handle(*apps)
    if filename:
        f = open(filename, "w")
        f.write(dump)
        f.close()
    else:
        return dump

def get_instance_url(model_instance, view_name):
    #TODO(nicoechaniz): this seems like a bad name. it returns the URL for an instance and for a non-instance as well. Also this code is repeated in many model files.
    view = cyclope.core.frontend.site.get_view(model_instance, view_name)

    if view.is_default:
        return '%s/%s/'\
                % (model_instance._meta.object_name.lower(),
                   model_instance.slug)

    if view.is_instance_view:
        return '%s/%s/View/%s'\
                % (model_instance._meta.object_name.lower(),
                   model_instance.slug, view_name)
    else:
        return '%s/View/%s'\
                % (model_instance._meta.object_name.lower(), view_name)


def get_content_urls(test_object):
    content_urls = []
    for view in frontend.site.get_views(test_object):
        if view.is_content_view:
            content_urls.append('/'+ get_instance_url(test_object, view.name))
    return content_urls


def get_region_views(test_model):
    return [ view for view in frontend.site.get_views(test_model)
             if view.is_region_view ]

def add_region_view(model, view_name, content_object=None):
    layout = get_default_layout()
    content_type = ContentType.objects.get(model=model._meta.module_name)
    content_view = view_name
    region = DEFAULT_THEME_REGION
    region_view = RegionView(layout=layout, content_type=content_type,
                             content_view=content_view, region=region,
                             content_object=content_object)
    region_view = region_view.save()
    return region_view


def get_default_site():
    if get_default_site.cache is None:
       get_default_site.cache = Site.objects.get_current()
    return get_default_site.cache

get_default_site.cache = None

def get_default_layout():
    if get_default_layout.cache is None:
       get_default_layout.cache = Layout.objects.all()[0]
    return get_default_layout.cache

get_default_layout.cache = None


class ViewableTestCase(TestCase):
    # this is an "abstract" test case which should be inherited but not run

#TODO (nicoechaniz): check how this base test class should be created to keep it out uf the test run

    fixtures = ['simplest_site.json']

    def setUp(self):
        if hasattr(self, 'test_model') and not hasattr(self, 'test_object'):
            self.test_object = self.test_model.objects.create(name='An instance')
        frontend.autodiscover()

    def test_creation(self):
        if hasattr(self, 'test_model'):
            an_instance = self.test_model.objects.get(slug='an-instance')
            self.assertEqual(an_instance.name, 'An instance')

    def test_content_views(self):
        if hasattr(self, 'test_object'):
            content_urls = get_content_urls(self.test_object)
            for url in content_urls:
                self.assertEqual(self.client.get(url).status_code, 200)

    def test_region_views(self):
        if hasattr(self, 'test_model'):
            model_region_views = get_region_views(self.test_model)
            for view in model_region_views:
                content_object = None
                if view.is_region_view:
                    content_object = self.test_object
                add_region_view(self.test_model, view.name, content_object)
                response = self.client.get("/")
                self.assertContains(response,
                                    'class="regionview %s %s"' %
                                    (self.test_model._meta.module_name, view.name),
                                    count=1)

class FrontendTestCase(TestCase):

    class Model(models.Model):
        name = "Test"

    class ModelDetail(frontend.FrontendView):
        name = 'model-detail'
        verbose_name = 'show a model'
        is_default = True
        is_instance_view = True
        is_region_view = False
        is_content_view = True

    def setUp(self):
        cyclope.core.frontend.site.register_view(self.Model, self.ModelDetail)

    def tearDown(self):
        cyclope.core.frontend.site.unregister_view(self.Model, self.ModelDetail)

    def test_register_view(self):
        view = cyclope.core.frontend.site.get_view(self.Model, 'model-detail')
        self.assertTrue(isinstance(view, self.ModelDetail))

    def test_get_view(self):
        view = cyclope.core.frontend.site.get_view(self.Model, 'model-detail')
        self.assertTrue(isinstance(view, self.ModelDetail))

        # Test with instance
        view = cyclope.core.frontend.site.get_view(self.Model(), 'model-detail')
        self.assertTrue(isinstance(view, self.ModelDetail))

    def test_get_views(self):
        views = cyclope.core.frontend.site.get_views(self.Model())
        self.assertEqual(len(views), 1)
        self.assertTrue(all([isinstance(v, self.ModelDetail) for v in views]))

    def test_unregister(self):
        cyclope.core.frontend.site.register_view(self.Model, self.ModelDetail)
        cyclope.core.frontend.site.unregister_view(self.Model, self.ModelDetail)
        views = cyclope.core.frontend.site.get_views(self.Model())
        self.assertEqual(len(views), 0)

    def test__name__expected(self):
        view = cyclope.core.frontend.site.get_view(self.Model, 'model-detail')
        self.assertEqual(view.__name__, view.name)



class SiteSettingsTestCase(TestCase):
    def setUp(self):
        self.site = Site(domain="mydomain.com", name="mydomain")
        self.site.save()

    def test_creation(self):
        site_settings = SiteSettings(site=self.site,
                                theme=DEFAULT_THEME,
                                allow_comments='YES')
        site_settings.save()
        self.assertEqual(site_settings.site, self.site)

    def test_force_not_deletion(self):
        col = Collection.objects.create(name='A collection')
        layout = Layout.objects.create(name="Test Layout", template='five_elements.html')

        site_settings = SiteSettings(site=self.site, theme=DEFAULT_THEME,
                                     allow_comments='YES',
                                     default_layout=layout,
                                     newsletter_collection=col)
        site_settings.save()

        # On self delete
        site_settings.delete()
        qs = SiteSettings.objects.filter(id=site_settings.id)
        self.assertTrue(len(qs) > 0)

        # On newsletter collection delete
        col.delete()
        qs = SiteSettings.objects.filter(id=site_settings.id)
        self.assertTrue(len(qs) > 0)

        # On layout collection delete
        layout.delete()
        qs = SiteSettings.objects.filter(id=site_settings.id)
        self.assertTrue(len(qs) > 0)


class SiteTestCase(TestCase):
    def testSimplestSite(self):
        """
        Test the simplest creation of a Cyclope-site.
        """
        # Simplest site should be created by syncdb
        response = self.client.get("/")
        self.assertTemplateUsed(response,
                                u'cyclope/themes/neutronix/five_elements.html')

    def testBugMenuItemWithoutLayout(self):
        # saving a MenuItem without setting a default site Layout failed
        site = get_default_site()
        menu = Menu.objects.get(main_menu=True)
        menu_item = MenuItem(menu=menu, name="without_layout", active=True)
        menu_item.save()

        site_settings = SiteSettings.objects.get(site=site)
        site_settings.save()
        response = self.client.get("/")


    def testSiteWithoutDefaultLayout(self):
        site = get_default_site()
        site_settings = SiteSettings.objects.get(site=site)
        site_settings.default_layout = None
        site_settings.save()
        response = self.client.get("/")
        self.assertEqual(response.content, 'You need to select a layout for the site')
        site_settings.default_layout = get_default_layout()
        site_settings.save()
        response = self.client.get("/")
        self.assertNotEqual(response.content, 'You need to select a layout for the site')
        #TODO(nicoechaniz): testing for the response content is weak; look for a better option
        # this view should use a standard error template and we should check that the template was used and some message id


    def testSiteWithoutHomeMenuitem(self):
        home = MenuItem.objects.get(site_home=True)
        home.site_home = False
        home.save()
        response = self.client.get("/")
        self.assertEqual(response.content, 'The site home page has not been set.')
        home.site_home = True
        home.save()
        response = self.client.get("/")
        self.assertNotEqual(response.content, 'The site home page has not been set.')
        #TODO(nicoechaniz): testing for the response content is weak; look for a better option


class RegressionTests(TestCase):

    def setUp(self):
        site = Site(domain="mydomain.com", name="mydomain")
        site.save()
        menu = Menu.objects.create(name="Main menu", main_menu=True)
        menu_item = MenuItem(menu=menu, name="home",
                                            site_home=True, active=True)
        menu_item.save()
        layout = Layout(name="default", template='five_elements.html')
        layout.save()
        site_settings = SiteSettings.objects.create(site=site,
                                theme=DEFAULT_THEME,
                                allow_comments='YES')
        site_settings.default_layout = layout
        site_settings.save()


class RegionViewTestCase(TestCase):
    fixtures = ['simplest_site.json']

    def testAddLayoutRegionView(self):
        layout = get_default_layout()
        content_type = ContentType.objects.get(model='staticpage')
        content_view = 'list'
        region = DEFAULT_THEME_REGION
        region_view = RegionView(layout=layout, content_type=content_type,
                                 content_view=content_view, region=region)
        region_view.save()
        response = self.client.get("/")
        self.assertContains(response, 'class="regionview staticpage list"', count=1)

    def testAddLayoutRegionViewInstanceViewWithoutContent(self):
        """If the view in a region needs a content object and none is provided
        a template error will be raised when visiting a page using this layout."""
        #TODO(nicoechaniz): this is prevented at admin form level, but should also be checked at data level. See note in forms.py RegionViewInlineForm.clean() method
        layout = get_default_layout()
        content_type = ContentType.objects.get(model='staticpage')
        content_view = 'detail'
        region = DEFAULT_THEME_REGION
        region_view = RegionView(layout=layout, content_type=content_type,
                                 content_view=content_view, region=region)
        region_view.save()
        self.assertRaises(TemplateSyntaxError, self.client.get, "/")


    def testAddLayoutRegionViewInstanceView(self):
        layout = get_default_layout()
        region_view = RegionView(layout=layout)
        content_type = ContentType.objects.get(model='staticpage')

        content_view = 'detail'
        static_page = create_static_page(
            name='test add layout region view instance view')
        object_id = static_page.id
        region = DEFAULT_THEME_REGION
        region_view = RegionView(
            layout=layout, content_type=content_type, content_view=content_view,
            object_id=object_id, region=region)
        region_view.save()
        response = self.client.get("/")
        self.assertContains(response, 'class="regionview staticpage detail',
                            count=1)


class TemplateTagsTestCase(TestCase):
    def setUp(self):
        register = template.Library()
        register.tag(name='join', compile_function=do_join)

    def test_join_strings(self):
        t = Template("{% load cyclope_utils %}"
                     "{% join 'Cy' 'clo' 'pe' as variable %}"
                     "{{ variable }}")
        c = Context({})
        response = t.render(c)
        self.assertEqual(response, "Cyclope")


class SiteMapViewTestCase(TestCase):
    fixtures = ['simplest_site.json']
    test_model = Site

    def test_creation(self):
        site = Site.objects.create(name='An instance')
        an_instance = Site.objects.get(name='An instance')
        self.assertEqual(an_instance.name, 'An instance')

    def test_site_map_view(self):
        site = Site.objects.create(name='site', domain='site')
        col = Collection.objects.create(name='A collection')
        cat = Category(name='An instance', collection=col)
        cat.save()
        menu = Menu.objects.create(name='menu')
        content_type = ContentType.objects.get(model='site')
        MenuItem(name='site map', menu=menu,
                 content_type=content_type, content_view="map").save()
        res = self.client.get('/site-map')
        self.assertEqual(res.status_code, 200)


class SiteSearchViewTestCase(TestCase):
    fixtures = ['default_users.json', 'default_groups.json', 'cyclope_demo.json']

    def setUp(self):
        frontend.autodiscover()

    def test_site_search_view(self):
        site = Site.objects.all()[0]
        search_url = ['/search/?q=cyclope']
        for model in frontend.site.base_content_types:
            model_query = "&models=%s.%s" % (model.get_app_label(), model.get_object_name())
            search_url.append(model_query)
        search_url = "".join(search_url)
        response = self.client.get(search_url)
        self.assertContains(response, 'id="search_results"', count=1)


class StaticPageTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = StaticPage


class AuthorTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Author

    def setUp(self):
        self.test_object = Author.objects.create(name="An instance")
        article = Article.objects.create(name='Article authored',
                                         author=self.test_object)
        frontend.autodiscover()

    def test_authored_content(self):
        content_urls = get_content_urls(self.test_object)
        for url in content_urls:
            response = self.client.get(url)
            self.assertContains(response, 'An instance')
            self.assertContains(response, 'Article authored')


class ArticleTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Article

    def setUp(self):
        author = Author.objects.create(name="the author")
        self.test_object = Article.objects.create(name='An instance',
                                                  author=author)
        frontend.autodiscover()


class DocumentTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Document


class ExternalContentTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = ExternalContent


class FlashMovieTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = FlashMovie

    def setUp(self):
        self.test_object = FlashMovie.objects.create(name='An instance', flash="/")
        frontend.autodiscover()


class MovieClipTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = MovieClip


class PictureTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Picture


class SoundTrackTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = SoundTrack


class PollTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Poll


class CategoryTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Category

    def setUp(self):
        # we need an authenticated user for some category views
        User = get_model('auth', 'user')
        self.user = User(username='admin')
        self.user.set_password('password')
        self.user.save()
        col = Collection.objects.create(name='A collection')
        self.test_object = Category(name='An instance', collection=col)
        self.test_object.save()
        frontend.autodiscover()


    def test_content_views(self):
        content_urls = get_content_urls(self.test_object)
        self.client.login(username='admin', password='password')
        for url in content_urls:
            self.assertEqual(self.client.get(url).status_code, 200)


class MoveCategoryTestCase(TestCase):
    def setUp(self):
        staticpage_ct = ContentType.objects.get(model="staticpage")
        collection_A = Collection.objects.create(name='collection A')
        collection_A.content_types.add(ContentType.objects.get(model="article"))
        collection_A.content_types.add(staticpage_ct)
        collection_A.save()

        collection_B = Collection.objects.create(name='collection B')
        collection_B.content_types.add(ContentType.objects.get(name="article"))
        collection_B.save()

        category = Category(name='Parent', collection=collection_A)
        category.save()

        child_category = Category(name='child', parent=category, collection=collection_A)
        child_category.save()

        children_of_child_category = Category(name='child child',
                                              parent=child_category,
                                              collection=collection_A)
        children_of_child_category.save()

        static_page = StaticPage.objects.create(name="static", text="prueba")
        static_page.categories.create(category=child_category)

        self.collection_A, self.collection_B = collection_A, collection_B
        self.category, self.child_category  = category, child_category
        self.child_of_child = children_of_child_category
        self.referesh_categories()

    def referesh_categories(self):
        self.category = Category.objects.get(id=self.category.id)
        self.child_category = Category.objects.get(id=self.child_category.id)
        self.child_of_child = Category.objects.get(id=self.child_of_child.id)

    def test_move_root_category_with_children(self):
        staticpage_ct = ContentType.objects.get(model="staticpage")

        self.category.collection = self.collection_B
        self.category.save()

        self.referesh_categories()

        # test change of collection in self and childs
        self.assertEqual(self.category.collection, self.collection_B)
        self.assertEqual(self.child_category.collection, self.collection_B)

        # test added proper content_types to the new collection
        self.assertTrue(staticpage_ct in self.category.collection.content_types.all())

    def test_move_child_category_with_children(self):
        self.assertEqual(self.child_category.get_root(), self.category)
        self.assertTrue(self.child_of_child in self.child_category.get_descendants())

        self.child_category.collection = self.collection_B
        self.child_category.save()

        self.referesh_categories()


        self.assertEqual(self.category.collection, self.collection_A)
        self.assertEqual(self.child_category.collection, self.collection_B)
        self.assertTrue(self.child_category.is_root_node())
        self.assertTrue(self.child_of_child in self.child_category.get_descendants())


class CollectionTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Collection

    def setUp(self):
        self.test_object = Collection.objects.create(name='An instance')
        self.test_object.save()
        # most views list categories in the collection, so we create one
        cat = Category(name='A Category', collection=self.test_object)
        cat.save()
        frontend.autodiscover()

    def test_get_widget_ajax(self):
        collection = Collection.objects.get(pk=1)
        response = self.client.get("/collection_categories_json",
                                  {"q": "1"})
        categories = json.loads(response.content)
        self.assertEqual(len(categories), 2)
        self.assertEqual(categories[0]["category_id"], "")
        self.assertEqual(categories[1]["category_id"], 1)
        self.assertContains(response, "A Category")


class MenuItemTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = MenuItem

    def setUp(self):
        self.menu = Menu.objects.create(name='menu')
        self.test_object = MenuItem(name='An instance', menu=self.menu)
        self.test_object.save()
        self.article = Article.objects.create(name='An article')
        frontend.autodiscover()

    def test_menu_item_without_view(self):
        self.test_object.content_object = self.article
        self.test_object.save()
        self.assertEqual(self.test_object.content_view,
                         frontend.site.get_default_view_name(Article))

    def test_menu_item_admin_form(self):

        # test without custom_url nor view/content
        self.assertTrue(self.build_admin_form().is_valid())

        # test with custom_url
        self.assertTrue(self.build_admin_form({'custom_url': "/foo/"}).is_valid())


        article_view = frontend.site.get_default_view_name(Article)
        article_ct_pk = ContentType.objects.get(model="article").pk
        # test with content_view
        data = {'content_view': article_view, 'content_type': article_ct_pk,
               'content_object': "%s-%s" % (article_ct_pk, self.article.pk)}
        self.assertTrue(self.build_admin_form(data).is_valid())

        # test fail: requires content_object
        data = {'content_view': article_view, 'content_type': article_ct_pk}
        self.assertFalse(self.build_admin_form(data).is_valid())

        # test fail: can't have custom_url and content
        data = {'custom_url': "/foo/", 'content_type': article_ct_pk}
        self.assertFalse(self.build_admin_form(data).is_valid())

    def test_active_item(self):
        # home menu and An instance menu items
        self.assertEqual(len(frontend.site.get_menuitem_urls()), 2)
        self.test_object.active = False
        self.test_object.save()
        self.assertEqual(len(frontend.site.get_menuitem_urls()), 1)

    def build_admin_form(self, new_data=None):
        base_data = {'menu':self.menu.pk, 'name': 'test_mi'}
        base_data.update(new_data or {})
        return MenuItemAdminForm(base_data)


class MenuTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Menu


class TopicTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Topic

    def setUp(self):
        User = get_model('auth', 'user')
        self.user = User(username='admin')
        self.user.set_password('password')
        self.user.save()
        self.test_object = Topic(name='An instance', user=self.user)
        self.test_object.save()
        frontend.autodiscover()


class FeedTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = Feed

    def setUp(self):
        self.test_object = Feed.objects.create(name="An instance",
                                               url="http://not.existant/rss")
        frontend.autodiscover()


class DynamicFormTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = DynamicForm

    def setUp(self):
        form = DynamicForm.objects.create(title="An instance")
        form.sites.add(Site.objects.get_current())
        form.save()
        form.fields.create(label="field", field_type=1, required=True, visible=True)
        self.test_object = form
        frontend.autodiscover()

    def test_empty_form(self):
        view = frontend.site.get_views(self.test_object)[0]
        url = '/'+ get_instance_url(self.test_object, view.name)
        response = self.client.post(url, data={})
        self.assertEqual(response.status_code, 200)


class MultipleFieldTestCase(TestCase):
    def setUp(self):
        class CategoryTeaserListOptions(forms.Form):
            items_per_page = forms.IntegerField(label='Items per page', initial=3, min_value=1)
            labeled = forms.BooleanField(label='Labeled', initial=False, required=False)

        class TestForm(forms.Form):
            field = MultipleField(form=CategoryTeaserListOptions())

        self.form = TestForm()

    def test_initial_values(self):
        self.assertIn('value="3"', self.form.as_p())

    def test_get_widget_ajax(self):
        response = self.client.get("/options_view_widget_html",
                                   {"content_type_name": "category",
                                    "view_name": "default"})
        self.assertContains(response, "view_options")


class CommentsViewsTestCase(ViewableTestCase):
    fixtures = ['simplest_site.json']
    test_model = django.contrib.comments.get_model()

    def setUp(self):
        site = Site.objects.get_current()
        comment = self.test_model(name="SAn", email="san@test.com", parent=None,
                                  content_object=site, site=site, subscribe=True)
        comment.save()
        self.test_object = comment
        frontend.autodiscover()

    def test_creation(self):
        pass

class UserProfileViewsTestCase(ViewableTestCase):
    test_model = UserProfile

    def setUp(self):
        user = User.objects.create_user('john', 'lennon@thebeatles.com', 'johnpassword')
        from registration import signals as registration_signals
        registration_signals.user_activated.send(sender=user, user=user)
        self.test_object = user.get_profile()
        frontend.autodiscover()

    def test_creation(self):
        pass

class DispatcherTestCase(TestCase):

    def test_unknown_url_returns_404(self):
        # Ticket https://trac.usla.org.ar/cyclope/ticket/43
        self.assertEqual(self.client.get("/category/foo/").status_code, 404)


class TestDemoFixture(TestCase):
    fixtures = ['default_users.json', 'default_groups.json', 'cyclope_demo.json']

    def test_demo_fixture(self):
        self.assertGreater(Category.objects.count(), 1)
        self.assertGreater(Collection.objects.count(), 2)
        self.assertGreater(MenuItem.objects.count(), 6)


class TestSitemaps(TestCase):

    fixtures = ['default_users.json', 'default_groups.json', 'cyclope_demo.json']
    sitemaps = [CollectionSitemap, CategorySitemap, MenuSitemap]
    longMessage = False

    @unittest.skip("this test fails when run on the test suite but not alone")
    def test_sitemap(self):
        for sitemap in self.sitemaps:
            sitemap = sitemap()
            urls = [obj.get("location") for obj in sitemap.get_urls()]
            for url in urls:
                response = self.client.get(url)
                status = response.status_code
                self.assertEqual(status, 200, "Broken url: %s, %d" % (url, status))


class ThemesTestCase(TestCase):

    def test_layout_form(self):
        form = LayoutAdminForm()
        choices = [choice[0] for choice in form.fields["template"].choices]
        self.assertTrue("five_elements.html" in choices)
        self.assertTrue("four_elements.html" in choices)

    def test_default_themes_integration(self):
        form = SiteSettingsAdminForm()
        choices = [choice[0] for choice in form.fields["theme"].choices]
        self.assertTrue(DEFAULT_THEME in choices)
        self.assertTrue("frecuency" in choices)

    @unittest.skip("this test fails when there is now custom_theme directory")
    def test_custom_theme_integration(self):
        form = SiteSettingsAdminForm()
        choices = [choice[0] for choice in form.fields["theme"].choices]
        self.assertTrue("custom_theme" in choices)

    def test_api(self):
        all_themes = themes.get_all_themes()
        theme = all_themes[DEFAULT_THEME]
        self.assertTrue("five_elements.html" in theme.layout_templates)
        self.assertTrue(theme is themes.get_theme(DEFAULT_THEME))


class MarkupTestCase(TestCase):

    def timeit(self, test_string):
        start = time.time()
        smart_style(test_string)
        interval = time.time() - start
        return interval

    def test_textile_hang(self):
        WAIT = 0.5
        cyclope_templatetags.cyclope_utils.MARKUP_RENDERER_WAIT = WAIT
        cyclope_templatetags.cyclope_utils.lru_cache.size_limit = 2

        foo_string = "foo" * 10000
        bar_string = "bar" * 10000

        self.assertTrue(self.timeit(foo_string) > WAIT) # non cached version
        self.assertTrue(self.timeit(foo_string) < WAIT) # cached version

        self.assertTrue(self.timeit(bar_string) > WAIT) # non cached version
        self.assertTrue(self.timeit(bar_string) < WAIT) # cached version

        self.assertTrue(self.timeit(foo_string) < WAIT) # cached version
        """
        for i in range(1, 5):
            print self.timeit("baz"*i*10000)
        for i in range(1, 5)[::-1]:
            print self.timeit("baz"*i*10000)
        """
        self.timeit("baz") # bar_string should fallen from the cache
        self.assertTrue(self.timeit(foo_string) > WAIT) # non cached version


class RelatedContentTestCase(TestCase):

    def setUp(self):
        self.article = Article.objects.create(name="Article")
        self.related_article = Article.objects.create(name="Article Related")

    def test_create_related(self):
        rc = RelatedContent.objects.create(self_object=self.article,
                                           other_object=self.related_article)
        self.assertEqual(self.article.related_contents.get().other_object,
                         self.related_article)

    def test_delete_relatedcontent_forward(self):
        """
        Tests deletion of RelatedContent objects when the self's related content
        is deleted.
        """
        rc = RelatedContent.objects.create(self_object=self.article,
                                           other_object=self.related_article)

        self.assertEqual(RelatedContent.objects.count(), 1)
        self.article.delete()
        self.assertEqual(RelatedContent.objects.count(), 0)

    def test_delete_relatedcontent_backward(self):
        """
        Tests deletion of RelatedContent objects when the other related content
        is deleted.
        """
        rc = RelatedContent.objects.create(self_object=self.article,
                                           other_object=self.related_article)

        self.assertEqual(RelatedContent.objects.count(), 1)
        self.related_article.delete()
        self.assertEqual(RelatedContent.objects.count(), 0)


class DeleteFromLayoutsAndMenuItems(TestCase):
    def setUp(self):
        MenuItem.objects.all().delete()

    def test_deletion(self):
        frontend.autodiscover()
        article = Article.objects.create(name="Article")

        layout = get_default_layout()
        region_view = RegionView(layout=layout)
        article_ct = ContentType.objects.get_for_model(article)
        region_view = RegionView.objects.create(content_object=article,
                                                layout=layout,
                                                content_view="detail")

        menu = Menu.objects.create(name='menu')
        mi = MenuItem(name='article item', menu=menu, content_object=article,
                      content_view="detail")
        mi.save()

        self.assertEqual(RegionView.objects.count(), 1)
        self.assertEqual(MenuItem.objects.get().content_object, article)
        article.delete()
        self.assertEqual(RegionView.objects.count(), 0)
        self.assertEqual(MenuItem.objects.get().content_object, None)


class FrontendEditTestCase(TestCase):
    fixtures = ['simplest_site.json']

    def setUp(self):
        self.article = Article.objects.create(name='Article')
        self.perm_user = User(username='perm_user', is_staff=True)
        self.perm_user.set_password('password')
        self.perm_user.save()
        self.non_perm_user = User(username='non_perm_user', is_staff=True)
        self.non_perm_user.set_password('password')
        self.non_perm_user.save()
        self.anonymous_user = AnonymousUser()
        col = Collection.objects.create(name='Collection')
        col.content_types.add(ContentType.objects.get(model="article"))
        col.save()
        self.category = Category(name='Category', collection=col)
        self.category.save()
        categorization = Categorization(category=self.category, content_object=self.article)
        self.article.categories.add(categorization)
        perm = CategoryPermission(user=self.perm_user, category=self.category,
                                  can_edit_content=True, can_add_content=True)
        perm.save()

        frontend.autodiscover()

    def test_add_content_perm(self):
        self.assertTrue(self.perm_user.has_perm('add_content', self.category))
        self.assertFalse(self.non_perm_user.has_perm('add_content', self.category))
        self.assertFalse(self.anonymous_user.has_perm('add_content', self.category))

    def test_edit_content_perm(self):
        self.assertTrue(self.perm_user.has_perm('edit_content', self.article))
        self.assertFalse(self.non_perm_user.has_perm('add_content', self.article))
        self.assertFalse(self.anonymous_user.has_perm('add_content', self.article))

    def test_edit_link(self):
        response = self.client.get('/article/article/')
        self.assertNotContains(response, 'class="edit_link"')

        self.client.login(username='perm_user', password='password')
        response = self.client.get('/article/article/')
        self.assertContains(response, 'class="edit_link"', count=1)

        self.client.login(username='non_perm_user', password='password')
        response = self.client.get('/article/article/')
        self.assertNotContains(response, 'class="edit_link"')

    def test_add_content_link(self):
        response = self.client.get('/category/category/')
        self.assertNotContains(response, 'class="category_add_content"')

        self.client.login(username=self.perm_user.username, password='password')
        response = self.client.get('/category/category/')
        self.assertContains(response, 'class="category_add_content"', count=1)

        self.client.login(username=self.non_perm_user.username, password='password')
        response = self.client.get('/category/category/')
        self.assertNotContains(response, 'class="category_add_content"')

class SimpleAdminTests(TestCase):
    """
    This is a realy simple test to catch errors on GET of some pages of the
    admin.
    """
    fixtures = ['default_users.json', 'default_groups.json', 'cyclope_demo.json']

    pages = [
        "/admin/",
        "/admin/cyclope/menuitem/", "/admin/cyclope/menuitem/1/",
        "/admin/cyclope/menu/", "/admin/cyclope/menu/1/",
        "/admin/cyclope/layout/", "/admin/cyclope/layout/1/",
        "/admin/collections/collection/", "/admin/collections/collection/1/",
        "/admin/collections/category/", "/admin/collections/category/1/",
        "/admin/articles/article/", "/admin/articles/article/1/",
        "/admin/cyclope/sitesettings/1/",
    ]

    def test_get_some_pages(self):
        admin = User.objects.get(username='admin')
        self.client.login(username='admin', password="password")
        for page in self.pages:
            status = self.client.get(page).status_code
            self.assertEqual(status, 200, "status: %d | page: %s" % (status, page))


class CategorizationManagerTests(TestCase):

    def test_get_for_category(self):
        """
        Tests that get_for_category returns the correct elements and making the correct
        number of queries.
        """
        col = Collection.objects.create(name='tema')
        article_ct = ContentType.objects.get(model="article")
        col.content_types.add(article_ct)
        col.save()
        category = Category.objects.create(name='Category', collection=col)
        for n in range(10):
            static_page = StaticPage.objects.create(name="static %d" % n, text="prueba"*100)
            static_page.categories.create(category=category)
            article = Article.objects.create(name="Test article %d" % n, text="prueba"*100)
            article.categories.create(category=category)

        self.assertNumQueries(4, Categorization.objects.get_for_category, category)

        cats = Categorization.objects.get_for_category(category, sort_property="creation_date", reverse=True)
        self.assertEqual(cats[0].content_object, Article.objects.latest("creation_date"))

        cats_random = Categorization.objects.get_for_category(category, sort_property="random")
        self.assertEqual(len(cats), len(cats_random))



class GetSingletonTests(TestCase):
    def test_get_singleton(self):
        from cyclope.utils import get_singleton
        site_settings = get_singleton(SiteSettings)
        site_settings.allow_comments = "YES"
        site_settings.save()
        site_settings = get_singleton(SiteSettings)
        self.assertEqual(site_settings.allow_comments, "YES")

        site_settings.allow_comments = "NO"
        site_settings.save()
        site_settings = get_singleton(SiteSettings)
        self.assertEqual(site_settings.allow_comments, "NO")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 Código Sur - Nuestra América Asoc. Civil / Fundación Pacificar.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ObjectDoesNotExist
from django.shortcuts import redirect
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse

from django.http import HttpResponseRedirect
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.utils.http import urlquote

from cyclope.core import frontend
from cyclope import views
from cyclope.core.collections.models import Category

from models import Topic
from forms import CreateTopicForm


class TopicDetail(frontend.FrontendView):
    name = 'detail'
    verbose_name= _('detailed view of the selected Topic')
    is_default = True
    is_instance_view = True
    is_content_view = True

    def get_response(self, request, req_context, options, content_object):
        avatar = None
        if content_object.user:
            try:
                profile = content_object.user.get_profile()
                avatar = profile.avatar
            except ObjectDoesNotExist:
                pass

        req_context.update({'avatar': avatar})
        return views.object_detail(request, req_context, content_object)

frontend.site.register_view(Topic, TopicDetail)

class CreateTopic(frontend.FrontendView):
    name = 'create_topic'
    verbose_name = _('create a new topic in the Forum selected')
    is_instance_view = True
    is_content_view = True

    def get_response(self, request, req_context, options, content_object):
        if not request.user.is_authenticated():
            from django.conf import settings
            tup = (settings.LOGIN_URL, REDIRECT_FIELD_NAME,
                   urlquote(request.get_full_path()))
            return HttpResponseRedirect('%s?%s=%s' % tup)
        category = content_object

        req_context.update({'forum': category.name})

        topic_ctype = ContentType.objects.get_for_model(Topic)
        if topic_ctype not in category.collection.content_types.all():
            not_allowed = True
            form = None
        else:
            not_allowed = False
            if request.method == 'POST':
                form = CreateTopicForm(data=request.POST)
                if form.is_valid():
                    # partial save
                    topic = form.save(commit=False)
                    topic.user = request.user
                    topic.allow_comments = 'YES'
                    topic.save()
                    # category added
                    topic.categories.get_or_create(category=category,
                                    defaults={'category': category,
                                              'content_type_id': topic_ctype.id,
                                              'object_id': topic.id})
                    topic.save()
                    return redirect(topic)
            else:
                form = CreateTopicForm()

        req_context.update({'form': form,
                            'not_allowed': not_allowed,
                            'action_url': reverse('category-create_topic',
                                               args=[category.slug]),
                            })

        return views.object_detail(request, req_context, content_object,
                                   view_name = self.name)

frontend.site.register_view(Category, CreateTopic)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2012 Código Sur Sociedad Civil.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.conf import settings
from django.core.urlresolvers import reverse
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _
from django.core.mail import mail_managers, send_mass_mail
from django.contrib.comments.managers import CommentManager

from threadedcomments import ThreadedComment

class CustomComment(ThreadedComment):

    subscribe = models.BooleanField(default=False)

    objects = CommentManager()

    def save(self, send_notifications=True, *args, **kwargs):

        created = not self.pk
        super(CustomComment, self).save(*args, **kwargs)

        if created and notification_enabled() and send_notifications:
            self.send_email_notifications()

    def send_email_notifications(self):
        subject = _("New comment posted on '%s'") % self.content_object
        message = self.get_as_text()
        managers_message = message
        if moderation_enabled():
            url = "http://%s%s" % (self.site.domain, reverse('comments-approve',
                                   args=(self.id,)))
            managers_message = _("This comment is moderated!\nYou may approve it " \
                                 "on the following url: %s\n\n") % url + message

        mail_managers(subject, managers_message, fail_silently=True)

        # Send mail to suscribed users of the tree path
        comments = CustomComment.objects.filter(id=self.root_path, subscribe=True)
        if comments:
            messages = [(subject, message, settings.DEFAULT_FROM_EMAIL,
                         [comment.userinfo["email"]]) for comment in comments]
            send_mass_mail(messages, fail_silently=True)

    class Meta:
        verbose_name = _("comment")
        verbose_name_plural = _("comments")



def moderation_enabled():
    from cyclope.utils import get_singleton # Fixme: move out of SiteSettings
    from cyclope.models import SiteSettings
    return get_singleton(SiteSettings).moderate_comments


def notification_enabled():
    from cyclope.utils import get_singleton # Fixme: move out of SiteSettings
    from cyclope.models import SiteSettings
    return get_singleton(SiteSettings).enable_comments_notifications

def comments_allowed():
    from cyclope.utils import get_singleton # Fixme: move out of SiteSettings
    from cyclope.models import SiteSettings
    return get_singleton(SiteSettings).allow_comments



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Código Sur Asoc. Civil
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
core.perms.admin
-------------------
"""

from django.contrib import admin
from cyclope.core.perms.models import CategoryPermission


class CategoryPermissionInline(admin.StackedInline):
    model = CategoryPermission
    extra = 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010-2012 Código Sur Sociedad Civil
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# urls should be set at project level using cyclope.site.autodiscover()
# to populate cyclope.site.urls

# see cyclope/demo for an example

from django.conf.urls import patterns, include, url
from django.conf import settings as django_settings
from django.contrib import admin
from haystack.views import SearchView, search_view_factory
from cyclope.core.captcha_contact_form.forms import  \
                                       AdminSettingsContactFormWithCaptcha
import cyclope.settings as cyc_settings
from cyclope.feeds import CategoryFeed, WholeSiteFeed, ContentTypeFeed
from cyclope.sitemaps import CategorySitemap, CollectionSitemap, MenuSitemap
from cyclope.core.user_profiles.forms import UserProfileForm

urlpatterns = patterns('',
    url(r'^sitemap\.xml$', 'django.contrib.sitemaps.views.sitemap',
        {'sitemaps': {"menus": MenuSitemap, "categories": CategorySitemap,
                      "collections": CollectionSitemap}}),
    url(r'^robots\.txt$', 'django.views.generic.simple.direct_to_template',
        {'template': 'cyclope/robots.txt', 'mimetype': 'text/plain'}),

    ## cyclope apps
    url(r'^locations/', include('cyclope.apps.locations.urls')),
    url(r'^newsletter/', include('cyclope.apps.newsletter.urls')),
    url(r'^abuse/', include('cyclope.apps.abuse.urls')),
    url(r'^related_admin/', include('cyclope.apps.related_admin.urls')),
    url(r'^rss/category/(?P<slug>[\w-]+)/$', CategoryFeed(), name='category_feed'),
    url(r'^rss/$', WholeSiteFeed(), name='whole_site_feed'),
    url(r'^rss/(?P<object_name>[\w-]+)/$', ContentTypeFeed(), name='content_type_feed'),
    url(r'^search/', search_view_factory(
        view_class=SearchView,
        results_per_page=cyc_settings.CYCLOPE_PAGINATION['TEASER']),
        name='haystack_search'),

    ## admin & django contrib
    url(r'^admin/filebrowser/', include('filebrowser.urls')),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^admin_tools/', include('admin_tools.urls')),
    url(r'^i18n/', include('django.conf.urls.i18n')),
    url(r'^comments/', include('django.contrib.comments.urls')),

    ## 3rd party
    # captcha (django-simple-captcha)
    url(r'^captcha/', include('captcha.urls')),
    # accounts (django-registration)
    url(r'^accounts/register/$', 'registration.views.register',
        {'backend': 'cyclope.registration_backends.CaptchaBackend'},
        name='registration_register'),
    url(r'^accounts/', include('registration.backends.default.urls')),

    # profiles (cyclope.core.user_profiles & django-profiles)
    url(r'^profiles/me/$', 'cyclope.core.user_profiles.views.me'), # to redirect to the proper url of user_profiles
    url(r'^profiles/create/$', 'profiles.views.create_profile',
        {'form_class': UserProfileForm, "success_url": "/profiles/me/"},
        name="profiles_create_profile"),
    url(r'^profiles/edit/$', 'profiles.views.edit_profile',
        {'form_class': UserProfileForm, "success_url": "/profiles/me/"},
        name="profiles_edit_profile"),
    url(r'^profiles/(?P<username>\w+)/$', "profiles.views.profile_detail",
        name='profiles_profile_detail'),

    # contact (django-contact-form)
    url(r'^contact/$', 'contact_form.views.contact_form',
        {'form_class': AdminSettingsContactFormWithCaptcha},
        name='contact_form'),
    url(r'^contact/', include('contact_form.urls')),
    # markitup
    url(r'^markitup/', include('markitup.urls')),
    # custom-forms (django-forms-builder)
    url(r'^forms/', include("forms_builder.forms.urls")),
    # django-generic-ratings
    (r'^ratings/', include('ratings.urls')),
)

if django_settings.DEBUG:
    urlpatterns+= patterns('',
        url(r'^media/(?P<path>.*)$', 'django.views.static.serve',
            {'document_root': django_settings.MEDIA_ROOT, 'show_indexes': True})
    )

if 'live' in django_settings.INSTALLED_APPS:
    urlpatterns += patterns('',
                            (r'^live/', include('live.urls')),
                            )

if 'schedule' in django_settings.INSTALLED_APPS:
    urlpatterns += patterns('',
                            (r'^schedule/', include('schedule.urls')),
                            )

if 'rosetta' in django_settings.INSTALLED_APPS:
    urlpatterns += patterns('',
        ## url(r'^rosetta/translate/(?P<langid>[\w\-]+)/(?P<idx>\d+)/',
        ##     'cyclope.helper_views.rosetta_select_and_translate'),
        url(r'^rosetta/', include('rosetta.urls')),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Código Sur Sociedad Civil.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
GenericModelForm  based on django-autocomplete-light
from James Pic and contributors.
https://github.com/yourlabs/django-autocomplete-light
"""

from django import forms
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericForeignKey


class GenericModelForm(forms.ModelForm):
    """
    This simple subclass of ModelForm fixes a couple of issues with django's
    ModelForm.

    - treat virtual fields like GenericForeignKey as normal fields, Django
      should already do that but it doesn't,
    - when setting a GenericForeignKey value, also set the object id and
      content type id fields, again Django could probably afford to do that.
    """
    def __init__(self, *args, **kwargs):
        """
        What ModelForm does, but also add virtual field values to self.initial.
        """
        super(GenericModelForm, self).__init__(*args, **kwargs)

        # do what model_to_dict doesn't
        for field in self._meta.model._meta.virtual_fields:
            self.initial[field.name] = getattr(self.instance, field.name, None)

    def _post_clean(self):
        """
        What ModelForm does, but also set virtual field values from
        cleaned_data.
        """
        super(GenericModelForm, self)._post_clean()
        # take care of virtual fields since django doesn't
        for field in self._meta.model._meta.virtual_fields:
            value = self.cleaned_data.get(field.name, None)

            if value:
                setattr(self.instance, field.name, value)

    def save(self, commit=True):
        """
        What ModelForm does, but also set GFK.ct_field and GFK.fk_field if such
        a virtual field has a value.

        This should probably be done in the GFK field itself, but it's here for
        convenience until Django fixes that.
        """
        for field in self._meta.model._meta.virtual_fields:
            if isinstance(field, GenericForeignKey):
                value = self.cleaned_data.get(field.name, None)

                if not value:
                    continue

                setattr(self.instance, field.ct_field,
                        ContentType.objects.get_for_model(value))
                setattr(self.instance, field.fk_field, value.pk)

        return super(GenericModelForm, self).save(commit)

    def clean(self):
        super(GenericModelForm, self).clean()
        # Add fk_fields of GenericFK to cleaned data (for inline forms)
        for field in self._meta.model._meta.virtual_fields:
            if isinstance(field, GenericForeignKey):
                value = self.cleaned_data.get(field.name, None)

                if not value:
                    continue
                self.cleaned_data[field.fk_field] = value.pk
        return self.cleaned_data




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Código Sur Sociedad Civil.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# urls should be set at project level using cyclope.site.autodiscover()
# to populate cyclope.site.urls

# see cyclope/demo for an example

"""URLs module"""
from django.conf.urls import patterns, url
from views import abuse_report

urlpatterns = patterns('',
    url(r'^report/(?P<ct_id>\d+)/(?P<obj_id>\d+)/', abuse_report,
        name="abuse-report"),
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'ExternalContent.author'
        db.add_column('medialibrary_externalcontent', 'author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cyclope.Author'], null=True, blank=True), keep_default=False)

        # Adding field 'RegularFile.author'
        db.add_column('medialibrary_regularfile', 'author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cyclope.Author'], null=True, blank=True), keep_default=False)

        # Adding field 'FlashMovie.author'
        db.add_column('medialibrary_flashmovie', 'author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cyclope.Author'], null=True, blank=True), keep_default=False)

        # Adding field 'Document.author'
        db.add_column('medialibrary_document', 'author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cyclope.Author'], null=True, blank=True), keep_default=False)

        # Adding field 'MovieClip.author'
        db.add_column('medialibrary_movieclip', 'author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cyclope.Author'], null=True, blank=True), keep_default=False)

        # Adding field 'SoundTrack.author'
        db.add_column('medialibrary_soundtrack', 'author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cyclope.Author'], null=True, blank=True), keep_default=False)

        # Adding field 'Picture.author'
        db.add_column('medialibrary_picture', 'author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cyclope.Author'], null=True, blank=True), keep_default=False)


    def backwards(self, orm):
        
        # Deleting field 'ExternalContent.author'
        db.delete_column('medialibrary_externalcontent', 'author_id')

        # Deleting field 'RegularFile.author'
        db.delete_column('medialibrary_regularfile', 'author_id')

        # Deleting field 'FlashMovie.author'
        db.delete_column('medialibrary_flashmovie', 'author_id')

        # Deleting field 'Document.author'
        db.delete_column('medialibrary_document', 'author_id')

        # Deleting field 'MovieClip.author'
        db.delete_column('medialibrary_movieclip', 'author_id')

        # Deleting field 'SoundTrack.author'
        db.delete_column('medialibrary_soundtrack', 'author_id')

        # Deleting field 'Picture.author'
        db.delete_column('medialibrary_picture', 'author_id')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'collections.categorization': {
            'Meta': {'object_name': 'Categorization'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'categorizations'", 'to': "orm['collections.Category']"}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'collections.category': {
            'Meta': {'unique_together': "(('collection', 'name'),)", 'object_name': 'Category'},
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'collection': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'categories'", 'to': "orm['collections.Collection']"}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '250', 'blank': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['collections.Category']"}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'collections.collection': {
            'Meta': {'object_name': 'Collection'},
            'content_types': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contenttypes.ContentType']", 'db_index': 'True', 'symmetrical': 'False'}),
            'default_list_view': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '250', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'navigation_root': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique_with': '()', 'max_length': '50', 'populate_from': 'None', 'db_index': 'True'}),
            'visible': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        'comments.comment': {
            'Meta': {'ordering': "('submit_date',)", 'object_name': 'Comment', 'db_table': "'django_comments'"},
            'comment': ('django.db.models.fields.TextField', [], {'max_length': '3000'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'content_type_set_for_comment'", 'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_address': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_removed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'object_pk': ('django.db.models.fields.TextField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'submit_date': ('django.db.models.fields.DateTimeField', [], {'default': 'None'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'comment_comments'", 'null': 'True', 'to': "orm['auth.User']"}),
            'user_email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'user_name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'user_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'cyclope.author': {
            'Meta': {'object_name': 'Author'},
            'content_types': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['contenttypes.ContentType']", 'db_index': 'True', 'symmetrical': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'default': "''", 'max_length': '100', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'}),
            'origin': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '250', 'db_index': 'True', 'blank': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'})
        },
        'cyclope.relatedcontent': {
            'Meta': {'ordering': "['order']", 'object_name': 'RelatedContent'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'other_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'other_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'related_contents_rt'", 'to': "orm['contenttypes.ContentType']"}),
            'self_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'self_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'related_contents_lt'", 'to': "orm['contenttypes.ContentType']"})
        },
        'medialibrary.document': {
            'Meta': {'object_name': 'Document'},
            'allow_comments': ('django.db.models.fields.CharField', [], {'default': "'SITE'", 'max_length': '4'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Author']", 'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704907)'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'document': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100', 'blank': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704976)', 'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),

            'txt_author': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'blank': 'True'})
        },
        'medialibrary.externalcontent': {
            'Meta': {'object_name': 'ExternalContent'},
            'allow_comments': ('django.db.models.fields.CharField', [], {'default': "'SITE'", 'max_length': '4'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Author']", 'null': 'True', 'blank': 'True'}),
            'content_url': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704907)'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100', 'blank': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704976)', 'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'new_window': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'skip_detail': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),

            'txt_author': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'blank': 'True'})
        },
        'medialibrary.flashmovie': {
            'Meta': {'object_name': 'FlashMovie'},
            'allow_comments': ('django.db.models.fields.CharField', [], {'default': "'SITE'", 'max_length': '4'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Author']", 'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704907)'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'flash': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100', 'blank': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704976)', 'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),

            'txt_author': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'blank': 'True'})
        },
        'medialibrary.movieclip': {
            'Meta': {'object_name': 'MovieClip'},
            'allow_comments': ('django.db.models.fields.CharField', [], {'default': "'SITE'", 'max_length': '4'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Author']", 'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704907)'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704976)', 'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),
            'still': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100', 'blank': 'True'}),

            'txt_author': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'blank': 'True'}),
            'video': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100'})
        },
        'medialibrary.picture': {
            'Meta': {'object_name': 'Picture'},
            'allow_comments': ('django.db.models.fields.CharField', [], {'default': "'SITE'", 'max_length': '4'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Author']", 'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704907)'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704976)', 'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),

            'txt_author': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'blank': 'True'})
        },
        'medialibrary.regularfile': {
            'Meta': {'object_name': 'RegularFile'},
            'allow_comments': ('django.db.models.fields.CharField', [], {'default': "'SITE'", 'max_length': '4'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Author']", 'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704907)'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'file': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('filebrowser.fields.FileBrowseField', [], {'max_length': '100', 'blank': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704976)', 'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),

            'txt_author': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'blank': 'True'})
        },
        'medialibrary.soundtrack': {
            'Meta': {'object_name': 'SoundTrack'},
            'allow_comments': ('django.db.models.fields.CharField', [], {'default': "'SITE'", 'max_length': '4'}),
            'audio': ('filebrowser.fields.FileBrowseField', [], {'max_length': '250'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cyclope.Author']", 'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704907)'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2011, 5, 2, 5, 56, 20, 704976)', 'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'db_index': 'True'}),
            'published': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None', 'unique_with': '()', 'db_index': 'True'}),

            'txt_author': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '100', 'blank': 'True'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['medialibrary']



from dbgettext.registry import registry, Options
from dbgettext.lexicons import html
from django.db.models import get_model

from models import Article

class ArticleOptions(Options):
    attributes = ('name',)
    parsed_attributes = {'text': html.lexicon, 'summary':html.lexicon}

    def get_path_identifier(self, obj):
        return obj.slug


registry.register(Article, ArticleOptions)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2012 Código Sur Sociedad Civil.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.comments.moderation import CommentModerator, moderator

import models

class CustomCommentModerator(CommentModerator):

    email_notification = False

    def moderate(self, comment, content_object, request):
        """
        Return ``True`` if the comment should be moderated (marked
        non-public), ``False`` otherwise.

        """
        if models.moderation_enabled():
            return True
        return False

    def allow(self, comment, content_object, request):
        """
        Return ``True`` if the comment should be allowed, ``False
        otherwise.

        """
        if models.comments_allowed() and \
            getattr(content_object, "allow_comments", True) != "NO":
            return True
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 - 2012 Código Sur Sociedad Civil
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""cyclope.frontend_views"""

from django import forms
from django.template import loader
from django.core.urlresolvers import reverse
from django.contrib.comments.models import Comment
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.core.paginator import Paginator

import cyclope.utils
from cyclope.frontend_views import MenuHierarchyOptions
from cyclope.utils import NamePaginator
from cyclope.core import frontend
from cyclope import settings as cyc_settings
from cyclope.core.collections.models import Collection, Category, Categorization

class CategoryRootItemsList(frontend.FrontendView):
    """A flat list view of the root members for a Category.
    """
    name='root_items_list'
    verbose_name=_('list of root items for the selected Category')

    is_content_view = True
    is_region_view = True

    def get_response(self, request, req_context, options, content_object):
        category = content_object
        categorizations_list = category.categorizations.all()
        template = "collections/category_root_items_list.html"
        req_context.update({"categorizations": categorizations_list,
                            "category": category})
        t = loader.get_template(template)
        return t.render(req_context)

frontend.site.register_view(Category, CategoryRootItemsList)

class TeaserListOptions(forms.Form):
    show_title = forms.BooleanField(label=_("Show title"), initial=True, required=False)
    show_description = forms.BooleanField(label=_("Show description"), initial=True, required=False)
    show_image = forms.BooleanField(label=_("Show image"), initial=True, required=False)
    items_per_page = forms.IntegerField(label=_('Items per page'), min_value=1,
                                        initial=cyc_settings.CYCLOPE_PAGINATION['TEASER'],)
    limit_to_n_items = forms.IntegerField(label=_('Limit to N items (0 = no limit)'),
                                          min_value=0, initial=0)
    sort_by = forms.ChoiceField(label=_('Sort by'),
                              choices=(("DATE-", _(u"Date ↓ (newest first)")),
                                       ("DATE+", _(u"Date ↑ (oldest first)")),
                                       ("ALPHABETIC", _(u"Alphabetic")),
                                       ("RANDOM", _(u"Random"))),
                              initial="DATE-")
    simplified = forms.BooleanField(label=_("Simplified"), initial=False, required=False)
    traverse_children = forms.BooleanField(label=_("Include descendant's elements"),
                                                    initial=False, required=False)
    navigation = forms.ChoiceField(label=_('Show navigation'),
                              choices=(("TOP", _(u"Top")),
                                       ("BOTTOM", _(u"Bottom")),
                                       ("DISABLED", _(u"Disabled"))),
                              initial="TOP")


class CategoryDefaultList(frontend.FrontendView):
    name = 'default'
    verbose_name = _('default view for the Collection')
    is_default = True
    is_content_view = True
    options_form = TeaserListOptions

    def get_response(self, request, req_context, options, content_object):
        category = content_object
        if category.collection.default_list_view not in ["", self.name]:
            view_name = category.collection.default_list_view
            view = frontend.site.get_view(content_object, view_name)
            if category.collection.view_options:
                options.update(category.collection.view_options)
            req_context['view_options'] = options
        else:
            view = frontend.site.get_view(content_object, 'teaser_list')
        return view.get_response(request, req_context, options, content_object)

frontend.site.register_view(Category, CategoryDefaultList)

SORT_BY = {
    "DATE-": ('creation_date', True, Paginator),
    "DATE+": ('creation_date', False, Paginator),
    "ALPHABETIC": ('name', None, NamePaginator),
    "RANDOM": ('random', False, Paginator),
}

def _get_paginator_page(category, options, request):
    traverse_children = options["traverse_children"]
    sort_by = options["sort_by"]
    limit = options["limit_to_n_items"] or None
    sort_property, reverse, paginator_class = SORT_BY[sort_by]
    search_args = [category, sort_property, limit, traverse_children]
    paginator, page = None, None
    if 'DATE' in sort_by:
        search_args.append(reverse)
    categorizations_list = Categorization.objects.get_for_category(*search_args)
    if options.get("items_per_page"):
        paginator_kwargs = {"per_page": options["items_per_page"]}
        if 'ALPHABETIC' in sort_by:
            paginator_kwargs['on'] = "content_object.name"
        paginator = paginator_class(categorizations_list, **paginator_kwargs)
        page = cyclope.utils.get_page(paginator, request)
    return categorizations_list, paginator, page

class CategoryTeaserList(frontend.FrontendView):
    """A teaser list view of Category members.
    """
    name = 'teaser_list'
    verbose_name = _('teaser list of Category members')
    is_content_view = True
    is_region_view = True
    options_form = TeaserListOptions
    inline_view_name = 'teaser'
    template = "collections/category_teaser_list.html"

    #TODO(nicoechaniz): find a more elegant way to build this view. current version is a temporary fix to a performance problem of the previous implementation when sorting big amounts of data.
    def get_response(self, request, req_context, options, content_object):
        category = content_object
        _, _, page = _get_paginator_page(category, options, request)
        req_context.update({'categorizations': page.object_list,
                            'page': page,
                            'category': category,
                            'inline_view_name': self.inline_view_name,
                            'simplified_view': options["simplified"]})
        t = loader.get_template(self.template)
        return t.render(req_context)

frontend.site.register_view(Category, CategoryTeaserList)


class CategoryLabeledIconList(CategoryTeaserList):
    """A labeled icon list view of Category members.
    """
    name='labeled_icon_list'
    verbose_name=_('labeled icon list of Category members')
    is_default = False
    items_per_page = cyc_settings.CYCLOPE_PAGINATION['LABELED_ICON']

    template = "collections/category_labeled_icon_list.html"
    inline_view_name = 'labeled_icon'

frontend.site.register_view(Category, CategoryLabeledIconList)


class SlideshowOptions(forms.Form):
    visualization_mode = forms.ChoiceField(label=_('Visual Mode'),
                              choices=(("slideshow", _(u"Standard view")),
                                       ("slideshow-background",
                                       _(u"With background image view"))),
                              initial="slideshow-background")
    show_title = forms.BooleanField(label=_("Show title"), initial=True, required=False)
    show_description = forms.BooleanField(label=_("Show description"), initial=True, required=False)
    show_image = forms.BooleanField(label=_("Show image"), initial=True, required=False)
    limit_to_n_items = forms.IntegerField(label=_('Limit to N items (0 = no limit)'),
                                          min_value=0, initial=10)
    sort_by = forms.ChoiceField(label=_('Sort by'),
                              choices=(("DATE-", _(u"Date ↓ (newest first)")),
                                       ("DATE+", _(u"Date ↑ (oldest first)")),
                                       ("ALPHABETIC", _(u"Alphabetic")),
                                       ("RANDOM", _(u"Random"))),
                              initial="DATE-")
    traverse_children = forms.BooleanField(label=_("Include descendant's elements"),
                                                    initial=False, required=False)
    scroll_by = forms.IntegerField(label=_('Scroll by N items'),
                               min_value=1, initial=3)
    speed = forms.IntegerField(label=_('Speed of transition (in milliseconds)'),
                               min_value=0, initial=1000)
    auto_play = forms.BooleanField(label=_("Automatic playback"),
                                   initial=True, required=False)
    auto_pause = forms.BooleanField(label=_("Stop automatic playback after click"),
                                    initial=True, required=False)
    delay = forms.IntegerField(label=_('Delay between transitions (seconds)'),
                               min_value=1, initial=3)
    circular = forms.BooleanField(label=_("Circular navigation"), initial=False,
                                  required=False)
    navigation = forms.ChoiceField(label=_('Navigation wrap'),
                              choices=(("'circular'", _(u"Circular")),
                                       ("'first'", _(u"First")),
                                       ("'last'", _(u"Last")),
                                       ("'both'", _(u"Both")),
                                       ("null", _(u"None"))),
                              initial="circular")

class CategorySlideshow(frontend.FrontendView):
    """A slideshow view of Category members.
    """
    name='slodeshow'
    verbose_name=_('slideshow of Category members')
    is_content_view = True
    is_region_view = True
    is_default = False
    options_form = SlideshowOptions
    inline_view_name = 'slideshow_item'
    template = "collections/category_slideshow.html"

    def get_response(self, request, req_context, options, content_object):
        category = content_object
        categorizations_list, _, _ = _get_paginator_page(category,
                                                         options,
                                                         request)
        req_context.update({'categorizations': categorizations_list,
                            'category': category,
                            'inline_view_name': self.inline_view_name,
                            'category_slug': category.slug
                            })
        t = loader.get_template(self.template)
        return t.render(req_context)

frontend.site.register_view(Category, CategorySlideshow)


class CategoryContents(CategoryTeaserList):
    """Full content of Category members.
    """
    name = 'contents'
    verbose_name = _('full content of Category members')
    items_per_page = cyc_settings.CYCLOPE_PAGINATION['DETAIL']
    is_content_view = True
    is_region_view = False

    template = "collections/category_contents.html"
    inline_view_name = 'inline_detail'

frontend.site.register_view(Category, CategoryContents)


class CollectionRootCategoriesTeaserList(frontend.FrontendView):
    """ A teaser list of the root Categories of a Collection
    """
    name = 'root_categories_teaser_list'
    verbose_name=_('teaser list of the root Categories of a Collection')
    is_default = True
    is_content_view = True
    is_region_view = True
    template = "collections/collection_root_categories_teaser_list.html"

    def get_response(self, request, req_context, options, content_object):
        collection = content_object
        req_context.update(
            {'categories': Category.tree.filter(collection=collection, level=0),
             'collection': collection })
        t = loader.get_template(self.template)
        return t.render(req_context)

frontend.site.register_view(Collection, CollectionRootCategoriesTeaserList)

class CollectionRootCategoriesList(CollectionRootCategoriesTeaserList):
    """ A list of the root Categories of a Collection
    """
    name = 'root_categories_list'
    verbose_name=_('list of the root Categories of a Collection')
    is_default = False
    is_region_view = True
    template = "collections/collection_root_categories_list.html"

frontend.site.register_view(Collection, CollectionRootCategoriesList)


class CollectionCategoriesHierarchy(frontend.FrontendView):
    """A hierarchical list view of the Categories in a Collection.
    """
    name='categories_hierarchy'
    verbose_name=_('hierarchical list of Categories in a Collection')
    target_view = 'default'
    is_content_view = True
    is_region_view = True
    options_form = MenuHierarchyOptions
    template = "collections/collection_categories_hierarchy.html"

    def get_response(self, request, req_context, options, content_object):
        collection = content_object
        categories = Category.tree.filter(collection=collection, level=0)
        category_list = []
        for category in categories:
            category_list.extend(self._get_categories_nested_list(category))
        req_context.update({'categories': category_list,
                            'collection_slug': collection.slug,
                            'align': options["align"]})
        t = loader.get_template(self.template)
        return t.render(req_context)

    def _get_categories_nested_list(self, base_category, name_field='name'):

        """Creates a nested list to be used with unordered_list template tag
        """
        #TODO(nicoechaniz): see if there's a more efficient way to build this recursive template data.
        from django.template import Template, Context
        link_template = Template(
              '<span class="{{ has_children }} {{ has_content}}">'
                '<a href="{% url category-'+ self.target_view +' slug %}">'
                 '{{ name }}</a>'
              '</span>'
            )

        def item_context(item):
            name = getattr(item, name_field)
            has_content = 'has_content' if item.categorizations.exists()\
                          else 'no_content'
            has_children = 'has_children' if item.get_descendant_count()\
                           else 'no_children'
            context = Context({'name': name,
                               'slug': item.slug,
                               'has_content': has_content,
                               'has_children': has_children,})
            return context

        nested_list = []
        for child in base_category.get_children():
            if child.get_descendant_count()>0:
                nested_list.extend(self._get_categories_nested_list(
                    child, name_field=name_field))
            else:
                nested_list.append(link_template.render(item_context(child)))

        include = link_template.render(item_context(base_category))
        if nested_list:
            return [include, nested_list]
        else:
            return [include]

frontend.site.register_view(Collection, CollectionCategoriesHierarchy)

class CollectionCategoriesHierarchyToIconlist(CollectionCategoriesHierarchy):
    """A hierarchical list view of the Categories in a Collection that will
    show a labeled_icon list view of the category that the user makes a
    selection.
    """
    name='categories_hierarchy_to_iconlist'
    verbose_name=_('hierarchical list of Categories that will show an icon list on click')
    target_view = 'labeled_icon_list'

frontend.site.register_view(Collection, CollectionCategoriesHierarchyToIconlist)

class CollectionCategoriesMembersAlphabeticTeaserList(frontend.FrontendView):
    """An alphabetic teaser list of all the members of the collection's categories.
    """
    name='collection_all_items_teaser_list'
    verbose_name=_("alphabetical teaser list of all the members of the Collection's Categories")
    is_content_view = True
    is_region_view = True
    items_per_page = cyc_settings.CYCLOPE_PAGINATION['TEASER']

    template = "collections/collection_all_items_alphabetical_teaser_list.html"

    def get_response(self, request, req_context, options, content_object):
        collection = content_object
        categorizations_list = Categorization.objects.filter(category__in=collection.categories.all())
        object_list = list(set([cat.content_object for cat in categorizations_list]))

        paginator = NamePaginator(object_list, on="name", per_page=self.items_per_page)
        page = cyclope.utils.get_page(paginator, request)

        req_context.update({'object_list': page.object_list,
                            'page': page,
                            'collection': collection
                            })
        t = loader.get_template(self.template)
        return t.render(req_context)

frontend.site.register_view(Collection, CollectionCategoriesMembersAlphabeticTeaserList)

class CategoryListAsForum(frontend.FrontendView):
    """ A list view of Category Members that will show a table with some extra
        information of the content (creation_date, count of comments, etc)
    """
    name='list_as_forum'
    verbose_name=_('list of Category members as forum view')
    is_default = False
    items_per_page = cyc_settings.CYCLOPE_PAGINATION['FORUM']
    is_content_view = True
    is_region_view = True

    template = "collections/category_list_as_forum.html"

    def get_response(self, request, req_context, options, content_object):
        category = content_object
        categorizations_list = Categorization.objects.get_for_category(
                                  category, 'modification_date', reverse=True)

        paginator = Paginator(categorizations_list, self.items_per_page)
        page = cyclope.utils.get_page(paginator, request)

        for categorization in page.object_list:
            obj = categorization.content_object
            ct = ContentType.objects.get_for_model(obj)
            qs = Comment.objects.filter(content_type=ct,object_pk=obj.pk)
            obj.comments_count = qs.count()
            obj.url = reverse(categorization.content_type.model, args=[obj.slug])
            if obj.comments_count:
                last_comment = qs.latest('submit_date')
                obj.last_comment_date = last_comment.submit_date
                obj.last_comment_author = last_comment.user_name

        req_context.update({'categorizations': page.object_list,
                            'page': page,
                            'category': category})
        t = loader.get_template(self.template)
        return t.render(req_context)

frontend.site.register_view(Category, CategoryListAsForum)



"""
nohang module provides a way to ensure that a function that could block get's
interrupted if it hangs more time that it should.
nohang does *not* support threading.
"""
import signal
import threading
import warnings

class TimeExpired(Exception):
    pass

_in_main_thread = isinstance(threading.current_thread(), threading._MainThread)
if not _in_main_thread:
   warnings.warn("nohang is running in a thread so it will not work", RuntimeWarning)


def run(function, args=None, kwargs=None, wait=None):
    """
    function gets called with args and kwargs and returns a tuple with the result
    and success. If function doesn't return in less than wait, in seconds (float),
    it's excecution is interrupted and the return value is (None, False).
    """

    if args is None:
        args = tuple()
    if kwargs is None:
        kwargs = dict()
    try:
        wait = float(wait)
    except:
        raise TypeError("wait argument should be a float")

    if _in_main_thread:
        signal.signal(signal.SIGALRM, _timer_handler)
        signal.setitimer(signal.ITIMER_REAL, wait)

    result, success = None, True
    try:
        result = function(*args, **kwargs)
    except TimeExpired:
        success = False

    if _in_main_thread:
        signal.setitimer(signal.ITIMER_REAL, 0) # Cancel the timer

    return result, success


def _timer_handler(signum, frame):
    raise TimeExpired






#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 Código Sur - Nuestra América Asoc. Civil / Fundación Pacificar.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _

from mptt.forms import TreeNodeChoiceField

from cyclope.apps.newsletter.models import Newsletter
from cyclope.core.collections.models import Category
from cyclope.models import SiteSettings
from cyclope.core.frontend.sites import site
from cyclope.utils import get_singleton

class NewsletterAdminForm(forms.ModelForm):
    content_category = TreeNodeChoiceField(
        queryset=Category.tree.all(), label=_('Current content category'),
        help_text=_('This is the category which groups the content that will be sent with the newsletter.'))
    view = forms.ChoiceField(label=_('View'))

    def __init__(self, *args, **kwargs):
        super(NewsletterAdminForm, self).__init__(*args, **kwargs)
        nl_collection = get_singleton(SiteSettings).newsletter_collection
        self.fields['content_category'].queryset = Category.tree.filter(collection=nl_collection)
        self.fields['content_category'].initial = Category.objects.latest()

        views = [('', '------')]
        views.extend([(view.name, view.verbose_name)
                       for view in site.get_views(Newsletter)
                       if view.is_content_view])
        self.fields['view'].choices = views

    class Meta:
        model = Newsletter


class NewsletterAdmin(admin.ModelAdmin):
    form = NewsletterAdminForm
    change_form_template = "admin/newsletter_change_form.html"

admin.site.register(Newsletter, NewsletterAdmin)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2012 Código Sur Sociedad Civil
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
LRU cache
---------
"""


try:
    from collections import OrderedDict
except ImportError:
    from cyclope.utils.ordereddict import OrderedDict


class LRULimitedSizeDict(OrderedDict):

    def __init__(self, *args, **kwds):
        self.size_limit = kwds.pop("size_limit", None)
        OrderedDict.__init__(self, *args, **kwds)
        self._check_size_limit()

    def __setitem__(self, key, value):
        OrderedDict.__setitem__(self, key, value)
        self._check_size_limit()

    def _check_size_limit(self):
        if self.size_limit is not None:
            while len(self) > self.size_limit:
                self.popitem(last=False)

    def __getitem__(self, key):
        value = OrderedDict.__getitem__(self, key)
        super(LRULimitedSizeDict, self).__delitem__(key)
        super(LRULimitedSizeDict, self).__setitem__(key, value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Código Sur Sociedad Civil.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from profiles.utils import get_profile_model

from cyclope.core import frontend
from cyclope.frontend_views import AuthoredMixin



class UserDetail(AuthoredMixin, frontend.FrontendView):
    """Display a user detail
    """
    name='detail'
    verbose_name=_('detailed view of the selected User')
    is_default = True
    is_instance_view = True
    is_region_view = False
    is_content_view = True
    template = "user_profiles/profile_detail.html"

    def get_queryset(self, content_object):
        qs = []
        # get related methods like picture_set, article_set, etc.
        # checking that are BaseContent types
        for related_name in dir(content_object):
            if related_name.endswith('_set'):
                rel_set = getattr(content_object, related_name)
                if rel_set.model in frontend.site.base_content_types:
                    sqs = rel_set.all()
                    # We only want the items that doesn't have author set
                    if hasattr(rel_set.model, "author"):
                        sqs = sqs.filter(author=None)
                    qs.extend(sqs)
        return qs

    def get_response(self, request, req_context, options, content_object):
        if options['show_authored_content']:
            authored_contents_page = self.get_page(request, req_context, options,
                                                   content_object.user)
        else:
            authored_contents_page = None

        return render_to_string(self.template, {
            'inline_view_name': options["inline_view_name"],
            'page': authored_contents_page
        }, req_context)

frontend.site.register_view(get_profile_model(), UserDetail)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
# Copyright 2012 Código Sur Sociedad Civil.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.utils.translation import ugettext_lazy as _

from threadedcomments.forms import ThreadedCommentForm
from captcha.fields import CaptchaField, CaptchaTextInput
from models import CustomComment


class CustomCaptchaTextInput(CaptchaTextInput):
    """
    This is the same as CaptchaTextInput but with 'deferred' image loading via
    javascript. The image html tag has a 'src_' attr instead of 'src'.
    """
    def render(self, name, value, attrs=None):
        ret = super(CustomCaptchaTextInput, self).render(name, value, attrs=None)
        return ret.replace("src=", "src_=")


class CustomCaptchaField(CaptchaField):

    def __init__(self, *args, **kwargs):
        super(CustomCaptchaField, self).__init__(*args, **kwargs)
        self.widget = CustomCaptchaTextInput(**{'output_format': u'%(image)s %(hidden_field)s %(text_field)s'})


class CustomCommentForm(ThreadedCommentForm):
    url = forms.URLField(label=_("Website or Blog"), required=False)
    captcha = CustomCaptchaField(label=_("Security code"))
    subscribe = forms.BooleanField(help_text=_('I want to be notified by email ' \
                                               'when there is a new comment.'),
                                   required=False)

    def clean(self):
        # captcha_ input only exists when the user is authenticated
        # this is not a real validation, submiting the form without the captcha_
        # will pass. The problem is that there's no way to know if the user us logged in
        # without rewriting a couple of views and template tags or use something like
        # django-contrib-requestprovider
        if 'captcha_' in self.data:
            if 'captcha' in self._errors:
                self._errors.pop('captcha')
        return super(CustomCommentForm, self).clean()

    def user_is_authenticated(self):
        del self.fields["captcha"]
        del self.fields["honeypot"]

    def get_comment_model(self):
        return CustomComment



from django.test import TestCase
from django.contrib.contenttypes.models import ContentType

from cyclope.core import frontend

from models import Series
from frontend_views import SeriesDetail
from django.test.client import RequestFactory

class SeriesTest(TestCase):
    def test_content_types(self):
        Series.content_models = [Series]
        ct = ContentType.objects.get_for_model(Series)
        self.assertEqual(Series.get_content_types(), [ct])
    
        self.assertEqual(Series.get_content_models(), [Series]) 
        
        self.assertEqual(Series.get_content_types_choices()[0], ("", '------'))
        self.assertEqual(Series.get_content_types_choices()[1], (ct.id, "series"))
    
    def test_detail_view(self):
        series_detail_view = SeriesDetail()
        series = Series(name="some_series")
        request = RequestFactory().get('/series/some_series')
        request.session = {}
        response = series_detail_view(request=request, content_object=series)
        self.assertContains(response, "some_series", status_code=200)
        self.assertTemplateUsed(response, "base_content_detail.html")
        
        
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2012 Código Sur Asoc. Civil
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
This app implements DynamicForms using django-forms-builder.
"""
from django.utils.translation import ugettext_lazy as _

from captcha.fields import CaptchaField

# Monkeypatching forms_builder.
# fields must me monkeypatched before of models!
from forms_builder.forms import fields as _fields

# Adding Captcha to field options
CAPTCHA = 101
_fields.CAPTCHA = CAPTCHA

_fields.NAMES = _fields.NAMES + ((CAPTCHA, _("Security code")),)

_fields.CLASSES[CAPTCHA] = CaptchaField

# now monkeypatching the model
from forms_builder.forms.models import Form as _Form

# Registered models must have a name attribute
_Form.name = property(lambda self:self.title)

_Form._meta.verbose_name = _('form')
_Form._meta.verbose_name_plural = _('forms')

DynamicForm = _Form



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010 Código Sur - Nuestra América Asoc. Civil / Fundación Pacificar.
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.utils.translation import ugettext_lazy as _

from autoslug.fields import AutoSlugField

from cyclope.models import BaseContent
from cyclope.core.collections.models import Collectible

class StaticPage(BaseContent, Collectible):
    summary = models.TextField(_('summary'), blank=True)
    text = models.TextField(_('text'))

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('static page')
        verbose_name_plural = _('static pages')


class HTMLBlock(models.Model):
    name = models.CharField(_('name'), max_length=250,
                             db_index=True, blank=False)
    slug = AutoSlugField(populate_from='name', unique=True,
                         db_index=True, always_update=True)    
    text = models.TextField(_('text'))
    
    def get_absolute_url(self):
        return '/%s/%s/' % (self._meta.object_name.lower(), self.slug)
    
    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('html block')
        verbose_name_plural = _('html blocks')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2010-2012 Código Sur Asociación Civil
# All rights reserved.
#
# This file is part of Cyclope.
#
# Cyclope is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cyclope is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


"""
apps.forum.models
--------------------
"""

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User

from cyclope.core.collections.models import Collectible

from cyclope.models import BaseContent


class Topic(BaseContent, Collectible):
    text = models.TextField(_('text'))

    class Meta:
        verbose_name = _('topic')
        verbose_name_plural = _('topics')
        ordering = ('-creation_date',)
